"""
Copyright (C) 2020-2022 Murilo Marques Marinho (www.murilomarinho.info)
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program. If not,
see <https://www.gnu.org/licenses/>.
"""
from umirobot.shared_memory.umirobot_shared_memory_common import shared_memory_map


class UMIRobotSharedMemoryProvider:
    def __init__(self, shared_memory_manager, lock, n_servos=6, n_potentiometers=6, n_digital_inputs=5):
        self.shared_memory_manager = shared_memory_manager
        self.lock = lock
        self.n_servos = n_servos
        self.n_potentiometers = n_potentiometers
        self.n_digital_inputs = n_digital_inputs
        self.connection_information_dict = shared_memory_map
        self.connection_information = shared_memory_manager.ShareableList(
            [n_servos,
             n_potentiometers,
             n_digital_inputs,
             False,
             "                                               ",
             False,
             False]
        )
        self.shareable_q = shared_memory_manager.ShareableList(
            [None] * n_servos
        )
        self.shareable_qd = shared_memory_manager.ShareableList(
            [None] * n_servos
        )
        self.shareable_potentiometer_values = shared_memory_manager.ShareableList(
            [None] * n_potentiometers
        )
        self.shareable_digital_in_values = shared_memory_manager.ShareableList(
            [None] * n_digital_inputs
        )

    def get_shared_memory_receiver_initializer_args(self):
        return (self.connection_information,
                self.shareable_q,
                self.shareable_qd,
                self.shareable_potentiometer_values,
                self.shareable_digital_in_values)

    def send_q(self, q):
        if q is not None:
            if len(q) == self.n_servos:
                self.lock.acquire()
                for i in range(0, self.n_servos):
                    self.shareable_q[i] = q[i]
                self.lock.release()

    def send_potentiometer_values(self, potentiometer_values):
        if potentiometer_values is not None:
            if len(potentiometer_values) == self.n_potentiometers:
                self.lock.acquire()
                for i in range(0, self.n_potentiometers):
                    self.shareable_potentiometer_values[i] = potentiometer_values[i]
                self.lock.release()

    def send_digital_in_values(self, digital_in_values):
        if digital_in_values is not None:
            if len(digital_in_values) == self.n_digital_inputs:
                self.lock.acquire()
                for i in range(0, self.n_digital_inputs):
                    self.shareable_digital_in_values[i] = digital_in_values[i]
                self.lock.release()

    def get_qd(self):
        self.lock.acquire()
        qd = list(self.shareable_qd)
        self.lock.release()
        return qd

    def send_is_open(self, is_open):
        self.lock.acquire()
        self.connection_information[self.connection_information_dict['is_open']] = is_open
        self.lock.release()

    def get_port(self):
        self.lock.acquire()
        port = self.connection_information[self.connection_information_dict['port']]
        self.lock.release()
        return port

    def get_port_connect_signal(self):
        self.lock.acquire()
        port_connect_signal = self.connection_information[self.connection_information_dict['port_connect_signal']]
        self.lock.release()
        return port_connect_signal

    def send_port_connect_signal(self, port_connect_signal):
        self.lock.acquire()
        self.connection_information[self.connection_information_dict['port_connect_signal']] = port_connect_signal
        self.lock.release()

    def get_shutdown_flag(self):
        self.lock.acquire()
        shutdown_flag = self.connection_information[self.connection_information_dict['shutdown_flag']]
        self.lock.release()
        return shutdown_flag

    def send_shutdown_flag(self, flag):
        self.lock.acquire()
        self.connection_information[self.connection_information_dict['shutdown_flag']] = flag
        self.lock.release()




# Form implementation generated from reading ui file 'umirobot_main_window.ui'
#
# Created by: PyQt6 UI code generator 6.3.1
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1197, 599)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("icon.png"), QtGui.QIcon.Mode.Normal, QtGui.QIcon.State.Off)
        MainWindow.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.frame_3 = QtWidgets.QFrame(self.centralwidget)
        self.frame_3.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame_3.setObjectName("frame_3")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.frame_3)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.frame_6 = QtWidgets.QFrame(self.frame_3)
        self.frame_6.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame_6.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame_6.setObjectName("frame_6")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.frame_6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_manual_control_angle_change = QtWidgets.QLabel(self.frame_6)
        self.label_manual_control_angle_change.setObjectName("label_manual_control_angle_change")
        self.verticalLayout.addWidget(self.label_manual_control_angle_change)
        self.slider_manual_control = QtWidgets.QSlider(self.frame_6)
        self.slider_manual_control.setMinimum(0)
        self.slider_manual_control.setMaximum(50)
        self.slider_manual_control.setProperty("value", 5)
        self.slider_manual_control.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.slider_manual_control.setTickPosition(QtWidgets.QSlider.TickPosition.TicksBothSides)
        self.slider_manual_control.setTickInterval(5)
        self.slider_manual_control.setObjectName("slider_manual_control")
        self.verticalLayout.addWidget(self.slider_manual_control)
        self.gridLayout_2.addWidget(self.frame_6, 4, 4, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.label_4 = QtWidgets.QLabel(self.frame_3)
        self.label_4.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout.addWidget(self.label_4)
        self.checkbox_jointcontrol = QtWidgets.QCheckBox(self.frame_3)
        self.checkbox_jointcontrol.setEnabled(True)
        self.checkbox_jointcontrol.setText("")
        self.checkbox_jointcontrol.setChecked(False)
        self.checkbox_jointcontrol.setObjectName("checkbox_jointcontrol")
        self.horizontalLayout.addWidget(self.checkbox_jointcontrol)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.gridLayout_2.addLayout(self.horizontalLayout, 1, 4, 1, 1)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.label_5 = QtWidgets.QLabel(self.frame_3)
        self.label_5.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.checkbox_teleoperation = QtWidgets.QCheckBox(self.frame_3)
        self.checkbox_teleoperation.setText("")
        self.checkbox_teleoperation.setObjectName("checkbox_teleoperation")
        self.horizontalLayout_2.addWidget(self.checkbox_teleoperation)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.gridLayout_2.addLayout(self.horizontalLayout_2, 1, 6, 1, 1)
        spacerItem4 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem4, 2, 5, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_2.addItem(spacerItem5, 4, 8, 1, 1)
        spacerItem6 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem6, 2, 3, 1, 1)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem7)
        self.label_6 = QtWidgets.QLabel(self.frame_3)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_3.addWidget(self.label_6)
        self.checkbox_potentiometer_cal = QtWidgets.QCheckBox(self.frame_3)
        self.checkbox_potentiometer_cal.setText("")
        self.checkbox_potentiometer_cal.setObjectName("checkbox_potentiometer_cal")
        self.horizontalLayout_3.addWidget(self.checkbox_potentiometer_cal)
        spacerItem8 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem8)
        self.gridLayout_2.addLayout(self.horizontalLayout_3, 1, 8, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.frame_3)
        self.label_2.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 1, 2, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.frame_3)
        self.label_7.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout_2.addWidget(self.label_7, 1, 0, 1, 1)
        self.frame_5 = QtWidgets.QFrame(self.frame_3)
        self.frame_5.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame_5.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame_5.setObjectName("frame_5")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.frame_5)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.button_increase_q_1 = QtWidgets.QPushButton(self.frame_5)
        self.button_increase_q_1.setObjectName("button_increase_q_1")
        self.gridLayout_3.addWidget(self.button_increase_q_1, 1, 1, 1, 1)
        self.button_decrease_q_4 = QtWidgets.QPushButton(self.frame_5)
        self.button_decrease_q_4.setObjectName("button_decrease_q_4")
        self.gridLayout_3.addWidget(self.button_decrease_q_4, 4, 0, 1, 1)
        self.button_decrease_q_5 = QtWidgets.QPushButton(self.frame_5)
        self.button_decrease_q_5.setObjectName("button_decrease_q_5")
        self.gridLayout_3.addWidget(self.button_decrease_q_5, 5, 0, 1, 1)
        self.button_decrease_q_1 = QtWidgets.QPushButton(self.frame_5)
        self.button_decrease_q_1.setObjectName("button_decrease_q_1")
        self.gridLayout_3.addWidget(self.button_decrease_q_1, 1, 0, 1, 1)
        self.button_increase_q_5 = QtWidgets.QPushButton(self.frame_5)
        self.button_increase_q_5.setObjectName("button_increase_q_5")
        self.gridLayout_3.addWidget(self.button_increase_q_5, 5, 1, 1, 1)
        self.button_decrease_q_3 = QtWidgets.QPushButton(self.frame_5)
        self.button_decrease_q_3.setObjectName("button_decrease_q_3")
        self.gridLayout_3.addWidget(self.button_decrease_q_3, 3, 0, 1, 1)
        self.button_decrease_q_0 = QtWidgets.QPushButton(self.frame_5)
        self.button_decrease_q_0.setObjectName("button_decrease_q_0")
        self.gridLayout_3.addWidget(self.button_decrease_q_0, 0, 0, 1, 1)
        self.button_increase_q_0 = QtWidgets.QPushButton(self.frame_5)
        self.button_increase_q_0.setObjectName("button_increase_q_0")
        self.gridLayout_3.addWidget(self.button_increase_q_0, 0, 1, 1, 1)
        self.button_decrease_q_2 = QtWidgets.QPushButton(self.frame_5)
        self.button_decrease_q_2.setObjectName("button_decrease_q_2")
        self.gridLayout_3.addWidget(self.button_decrease_q_2, 2, 0, 1, 1)
        self.button_increase_q_2 = QtWidgets.QPushButton(self.frame_5)
        self.button_increase_q_2.setObjectName("button_increase_q_2")
        self.gridLayout_3.addWidget(self.button_increase_q_2, 2, 1, 1, 1)
        self.button_increase_q_3 = QtWidgets.QPushButton(self.frame_5)
        self.button_increase_q_3.setObjectName("button_increase_q_3")
        self.gridLayout_3.addWidget(self.button_increase_q_3, 3, 1, 1, 1)
        self.button_increase_q_4 = QtWidgets.QPushButton(self.frame_5)
        self.button_increase_q_4.setObjectName("button_increase_q_4")
        self.gridLayout_3.addWidget(self.button_increase_q_4, 4, 1, 1, 1)
        self.gridLayout_2.addWidget(self.frame_5, 2, 4, 2, 1)
        spacerItem9 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem9, 3, 1, 1, 1)
        self.frame = QtWidgets.QFrame(self.frame_3)
        self.frame.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame.setObjectName("frame")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.frame)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.lineedit_q_2 = QtWidgets.QLineEdit(self.frame)
        self.lineedit_q_2.setEnabled(False)
        self.lineedit_q_2.setObjectName("lineedit_q_2")
        self.gridLayout_6.addWidget(self.lineedit_q_2, 2, 1, 1, 1)
        self.lineedit_q_0 = QtWidgets.QLineEdit(self.frame)
        self.lineedit_q_0.setEnabled(False)
        self.lineedit_q_0.setObjectName("lineedit_q_0")
        self.gridLayout_6.addWidget(self.lineedit_q_0, 0, 1, 1, 1)
        self.label_q_0 = QtWidgets.QLabel(self.frame)
        self.label_q_0.setObjectName("label_q_0")
        self.gridLayout_6.addWidget(self.label_q_0, 0, 0, 1, 1)
        self.label_q_1 = QtWidgets.QLabel(self.frame)
        self.label_q_1.setObjectName("label_q_1")
        self.gridLayout_6.addWidget(self.label_q_1, 1, 0, 1, 1)
        self.lineedit_q_4 = QtWidgets.QLineEdit(self.frame)
        self.lineedit_q_4.setEnabled(False)
        self.lineedit_q_4.setObjectName("lineedit_q_4")
        self.gridLayout_6.addWidget(self.lineedit_q_4, 4, 1, 1, 1)
        self.lineedit_q_3 = QtWidgets.QLineEdit(self.frame)
        self.lineedit_q_3.setEnabled(False)
        self.lineedit_q_3.setObjectName("lineedit_q_3")
        self.gridLayout_6.addWidget(self.lineedit_q_3, 3, 1, 1, 1)
        self.label_q_2 = QtWidgets.QLabel(self.frame)
        self.label_q_2.setObjectName("label_q_2")
        self.gridLayout_6.addWidget(self.label_q_2, 2, 0, 1, 1)
        self.label_q_3 = QtWidgets.QLabel(self.frame)
        self.label_q_3.setObjectName("label_q_3")
        self.gridLayout_6.addWidget(self.label_q_3, 3, 0, 1, 1)
        self.lineedit_q_1 = QtWidgets.QLineEdit(self.frame)
        self.lineedit_q_1.setEnabled(False)
        self.lineedit_q_1.setObjectName("lineedit_q_1")
        self.gridLayout_6.addWidget(self.lineedit_q_1, 1, 1, 1, 1)
        self.label_q_4 = QtWidgets.QLabel(self.frame)
        self.label_q_4.setObjectName("label_q_4")
        self.gridLayout_6.addWidget(self.label_q_4, 4, 0, 1, 1)
        self.label_q_5 = QtWidgets.QLabel(self.frame)
        self.label_q_5.setObjectName("label_q_5")
        self.gridLayout_6.addWidget(self.label_q_5, 5, 0, 1, 1)
        self.lineedit_q_5 = QtWidgets.QLineEdit(self.frame)
        self.lineedit_q_5.setEnabled(False)
        self.lineedit_q_5.setObjectName("lineedit_q_5")
        self.gridLayout_6.addWidget(self.lineedit_q_5, 5, 1, 1, 1)
        self.gridLayout_2.addWidget(self.frame, 2, 2, 2, 1)
        self.frame_7 = QtWidgets.QFrame(self.frame_3)
        self.frame_7.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame_7.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame_7.setObjectName("frame_7")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.frame_7)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.lineedit_pot_min_1 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_min_1.setEnabled(False)
        self.lineedit_pot_min_1.setObjectName("lineedit_pot_min_1")
        self.gridLayout_7.addWidget(self.lineedit_pot_min_1, 1, 0, 1, 1)
        self.lineedit_pot_max_3 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_max_3.setEnabled(False)
        self.lineedit_pot_max_3.setObjectName("lineedit_pot_max_3")
        self.gridLayout_7.addWidget(self.lineedit_pot_max_3, 3, 2, 1, 1)
        self.lineedit_pot_max_1 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_max_1.setEnabled(False)
        self.lineedit_pot_max_1.setObjectName("lineedit_pot_max_1")
        self.gridLayout_7.addWidget(self.lineedit_pot_max_1, 1, 2, 1, 1)
        self.lineedit_pot_min_2 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_min_2.setEnabled(False)
        self.lineedit_pot_min_2.setObjectName("lineedit_pot_min_2")
        self.gridLayout_7.addWidget(self.lineedit_pot_min_2, 2, 0, 1, 1)
        self.lineedit_pot_min_4 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_min_4.setEnabled(False)
        self.lineedit_pot_min_4.setObjectName("lineedit_pot_min_4")
        self.gridLayout_7.addWidget(self.lineedit_pot_min_4, 4, 0, 1, 1)
        self.lineedit_pot_min_0 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_min_0.setEnabled(False)
        self.lineedit_pot_min_0.setObjectName("lineedit_pot_min_0")
        self.gridLayout_7.addWidget(self.lineedit_pot_min_0, 0, 0, 1, 1)
        self.lineedit_pot_min_3 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_min_3.setEnabled(False)
        self.lineedit_pot_min_3.setObjectName("lineedit_pot_min_3")
        self.gridLayout_7.addWidget(self.lineedit_pot_min_3, 3, 0, 1, 1)
        self.lineedit_pot_max_4 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_max_4.setEnabled(False)
        self.lineedit_pot_max_4.setObjectName("lineedit_pot_max_4")
        self.gridLayout_7.addWidget(self.lineedit_pot_max_4, 4, 2, 1, 1)
        self.lineedit_pot_max_0 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_max_0.setEnabled(False)
        self.lineedit_pot_max_0.setObjectName("lineedit_pot_max_0")
        self.gridLayout_7.addWidget(self.lineedit_pot_max_0, 0, 2, 1, 1)
        self.radio_pot_0 = QtWidgets.QRadioButton(self.frame_7)
        self.radio_pot_0.setEnabled(False)
        self.radio_pot_0.setText("")
        self.radio_pot_0.setAutoExclusive(False)
        self.radio_pot_0.setObjectName("radio_pot_0")
        self.gridLayout_7.addWidget(self.radio_pot_0, 0, 3, 1, 1)
        self.lineedit_pot_max_5 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_max_5.setEnabled(False)
        self.lineedit_pot_max_5.setObjectName("lineedit_pot_max_5")
        self.gridLayout_7.addWidget(self.lineedit_pot_max_5, 5, 2, 1, 1)
        self.lineedit_pot_min_5 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_min_5.setEnabled(False)
        self.lineedit_pot_min_5.setObjectName("lineedit_pot_min_5")
        self.gridLayout_7.addWidget(self.lineedit_pot_min_5, 5, 0, 1, 1)
        self.radio_pot_1 = QtWidgets.QRadioButton(self.frame_7)
        self.radio_pot_1.setEnabled(False)
        self.radio_pot_1.setText("")
        self.radio_pot_1.setAutoExclusive(False)
        self.radio_pot_1.setObjectName("radio_pot_1")
        self.gridLayout_7.addWidget(self.radio_pot_1, 1, 3, 1, 1)
        self.lineedit_pot_max_2 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_max_2.setEnabled(False)
        self.lineedit_pot_max_2.setObjectName("lineedit_pot_max_2")
        self.gridLayout_7.addWidget(self.lineedit_pot_max_2, 2, 2, 1, 1)
        self.radio_pot_2 = QtWidgets.QRadioButton(self.frame_7)
        self.radio_pot_2.setEnabled(False)
        self.radio_pot_2.setText("")
        self.radio_pot_2.setAutoExclusive(False)
        self.radio_pot_2.setObjectName("radio_pot_2")
        self.gridLayout_7.addWidget(self.radio_pot_2, 2, 3, 1, 1)
        self.radio_pot_3 = QtWidgets.QRadioButton(self.frame_7)
        self.radio_pot_3.setEnabled(False)
        self.radio_pot_3.setText("")
        self.radio_pot_3.setAutoExclusive(False)
        self.radio_pot_3.setObjectName("radio_pot_3")
        self.gridLayout_7.addWidget(self.radio_pot_3, 3, 3, 1, 1)
        self.radio_pot_4 = QtWidgets.QRadioButton(self.frame_7)
        self.radio_pot_4.setEnabled(False)
        self.radio_pot_4.setText("")
        self.radio_pot_4.setAutoExclusive(False)
        self.radio_pot_4.setObjectName("radio_pot_4")
        self.gridLayout_7.addWidget(self.radio_pot_4, 4, 3, 1, 1)
        self.radio_pot_5 = QtWidgets.QRadioButton(self.frame_7)
        self.radio_pot_5.setEnabled(False)
        self.radio_pot_5.setText("")
        self.radio_pot_5.setAutoExclusive(False)
        self.radio_pot_5.setObjectName("radio_pot_5")
        self.gridLayout_7.addWidget(self.radio_pot_5, 5, 3, 1, 1)
        self.lineedit_pot_0 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_0.setEnabled(False)
        self.lineedit_pot_0.setObjectName("lineedit_pot_0")
        self.gridLayout_7.addWidget(self.lineedit_pot_0, 0, 1, 1, 1)
        self.lineedit_pot_1 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_1.setEnabled(False)
        self.lineedit_pot_1.setObjectName("lineedit_pot_1")
        self.gridLayout_7.addWidget(self.lineedit_pot_1, 1, 1, 1, 1)
        self.lineedit_pot_2 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_2.setEnabled(False)
        self.lineedit_pot_2.setObjectName("lineedit_pot_2")
        self.gridLayout_7.addWidget(self.lineedit_pot_2, 2, 1, 1, 1)
        self.lineedit_pot_3 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_3.setEnabled(False)
        self.lineedit_pot_3.setObjectName("lineedit_pot_3")
        self.gridLayout_7.addWidget(self.lineedit_pot_3, 3, 1, 1, 1)
        self.lineedit_pot_4 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_4.setEnabled(False)
        self.lineedit_pot_4.setObjectName("lineedit_pot_4")
        self.gridLayout_7.addWidget(self.lineedit_pot_4, 4, 1, 1, 1)
        self.lineedit_pot_5 = QtWidgets.QLineEdit(self.frame_7)
        self.lineedit_pot_5.setEnabled(False)
        self.lineedit_pot_5.setObjectName("lineedit_pot_5")
        self.gridLayout_7.addWidget(self.lineedit_pot_5, 5, 1, 1, 1)
        self.gridLayout_2.addWidget(self.frame_7, 2, 8, 2, 1)
        spacerItem10 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_2.addItem(spacerItem10, 4, 2, 1, 1)
        self.label_image = QtWidgets.QLabel(self.frame_3)
        self.label_image.setText("")
        self.label_image.setPixmap(QtGui.QPixmap("umirobot.png"))
        self.label_image.setObjectName("label_image")
        self.gridLayout_2.addWidget(self.label_image, 3, 0, 1, 1)
        self.frame_4 = QtWidgets.QFrame(self.frame_3)
        self.frame_4.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame_4.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame_4.setObjectName("frame_4")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.frame_4)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.label_pot_0 = QtWidgets.QLabel(self.frame_4)
        self.label_pot_0.setObjectName("label_pot_0")
        self.gridLayout_5.addWidget(self.label_pot_0, 0, 0, 1, 2)
        self.lineedit_teleop_1 = QtWidgets.QLineEdit(self.frame_4)
        self.lineedit_teleop_1.setEnabled(False)
        self.lineedit_teleop_1.setObjectName("lineedit_teleop_1")
        self.gridLayout_5.addWidget(self.lineedit_teleop_1, 1, 2, 1, 1)
        self.lineedit_teleop_5 = QtWidgets.QLineEdit(self.frame_4)
        self.lineedit_teleop_5.setEnabled(False)
        self.lineedit_teleop_5.setObjectName("lineedit_teleop_5")
        self.gridLayout_5.addWidget(self.lineedit_teleop_5, 5, 2, 1, 1)
        self.label_pot_2 = QtWidgets.QLabel(self.frame_4)
        self.label_pot_2.setObjectName("label_pot_2")
        self.gridLayout_5.addWidget(self.label_pot_2, 2, 0, 1, 1)
        self.label_pot_3 = QtWidgets.QLabel(self.frame_4)
        self.label_pot_3.setObjectName("label_pot_3")
        self.gridLayout_5.addWidget(self.label_pot_3, 3, 0, 1, 1)
        self.label_pot_5 = QtWidgets.QLabel(self.frame_4)
        self.label_pot_5.setObjectName("label_pot_5")
        self.gridLayout_5.addWidget(self.label_pot_5, 5, 0, 1, 1)
        self.lineedit_teleop_2 = QtWidgets.QLineEdit(self.frame_4)
        self.lineedit_teleop_2.setEnabled(False)
        self.lineedit_teleop_2.setObjectName("lineedit_teleop_2")
        self.gridLayout_5.addWidget(self.lineedit_teleop_2, 2, 2, 1, 1)
        self.lineedit_teleop_3 = QtWidgets.QLineEdit(self.frame_4)
        self.lineedit_teleop_3.setEnabled(False)
        self.lineedit_teleop_3.setObjectName("lineedit_teleop_3")
        self.gridLayout_5.addWidget(self.lineedit_teleop_3, 3, 2, 1, 1)
        self.lineedit_teleop_0 = QtWidgets.QLineEdit(self.frame_4)
        self.lineedit_teleop_0.setEnabled(False)
        self.lineedit_teleop_0.setObjectName("lineedit_teleop_0")
        self.gridLayout_5.addWidget(self.lineedit_teleop_0, 0, 2, 1, 1)
        self.label_pot_1 = QtWidgets.QLabel(self.frame_4)
        self.label_pot_1.setObjectName("label_pot_1")
        self.gridLayout_5.addWidget(self.label_pot_1, 1, 0, 1, 2)
        self.label_pot_4 = QtWidgets.QLabel(self.frame_4)
        self.label_pot_4.setObjectName("label_pot_4")
        self.gridLayout_5.addWidget(self.label_pot_4, 4, 0, 1, 1)
        self.lineedit_teleop_4 = QtWidgets.QLineEdit(self.frame_4)
        self.lineedit_teleop_4.setEnabled(False)
        self.lineedit_teleop_4.setObjectName("lineedit_teleop_4")
        self.gridLayout_5.addWidget(self.lineedit_teleop_4, 4, 2, 1, 1)
        self.gridLayout_2.addWidget(self.frame_4, 2, 6, 2, 1)
        spacerItem11 = QtWidgets.QSpacerItem(0, 0, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_2.addItem(spacerItem11, 2, 7, 1, 1)
        spacerItem12 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_2.addItem(spacerItem12, 4, 6, 1, 1)
        self.gridLayout_9.addWidget(self.frame_3, 0, 0, 1, 3)
        self.label_13 = QtWidgets.QLabel(self.centralwidget)
        self.label_13.setObjectName("label_13")
        self.gridLayout_9.addWidget(self.label_13, 1, 0, 1, 1)
        self.frame_10 = QtWidgets.QFrame(self.centralwidget)
        self.frame_10.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame_10.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame_10.setObjectName("frame_10")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.frame_10)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        spacerItem13 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem13)
        self.checkbox_communication_arduino_all = QtWidgets.QCheckBox(self.frame_10)
        self.checkbox_communication_arduino_all.setChecked(False)
        self.checkbox_communication_arduino_all.setObjectName("checkbox_communication_arduino_all")
        self.horizontalLayout_6.addWidget(self.checkbox_communication_arduino_all)
        spacerItem14 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem14)
        self.checkbox_communication_arduino_master_only = QtWidgets.QCheckBox(self.frame_10)
        self.checkbox_communication_arduino_master_only.setObjectName("checkbox_communication_arduino_master_only")
        self.horizontalLayout_6.addWidget(self.checkbox_communication_arduino_master_only)
        spacerItem15 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem15)
        self.gridLayout_4.addLayout(self.horizontalLayout_6, 1, 0, 1, 1)
        self.frame_9 = QtWidgets.QFrame(self.frame_10)
        self.frame_9.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame_9.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame_9.setObjectName("frame_9")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.frame_9)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.label_11 = QtWidgets.QLabel(self.frame_9)
        self.label_11.setObjectName("label_11")
        self.gridLayout_8.addWidget(self.label_11, 0, 0, 1, 1)
        self.lineedit_vrep_ip = QtWidgets.QLineEdit(self.frame_9)
        self.lineedit_vrep_ip.setMinimumSize(QtCore.QSize(150, 0))
        self.lineedit_vrep_ip.setObjectName("lineedit_vrep_ip")
        self.gridLayout_8.addWidget(self.lineedit_vrep_ip, 0, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.frame_9)
        self.label_10.setObjectName("label_10")
        self.gridLayout_8.addWidget(self.label_10, 1, 0, 1, 1)
        self.lineedit_vrep_port = QtWidgets.QLineEdit(self.frame_9)
        self.lineedit_vrep_port.setObjectName("lineedit_vrep_port")
        self.gridLayout_8.addWidget(self.lineedit_vrep_port, 1, 1, 1, 1)
        self.label_connection_status_vrep = QtWidgets.QLabel(self.frame_9)
        self.label_connection_status_vrep.setObjectName("label_connection_status_vrep")
        self.gridLayout_8.addWidget(self.label_connection_status_vrep, 2, 0, 1, 2)
        self.button_connect_vrep = QtWidgets.QPushButton(self.frame_9)
        self.button_connect_vrep.setObjectName("button_connect_vrep")
        self.gridLayout_8.addWidget(self.button_connect_vrep, 3, 0, 1, 2)
        self.gridLayout_4.addWidget(self.frame_9, 2, 2, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem16 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem16)
        self.label_9 = QtWidgets.QLabel(self.frame_10)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_5.addWidget(self.label_9)
        self.checkbox_communication_coppeliasim = QtWidgets.QCheckBox(self.frame_10)
        self.checkbox_communication_coppeliasim.setText("")
        self.checkbox_communication_coppeliasim.setObjectName("checkbox_communication_coppeliasim")
        self.horizontalLayout_5.addWidget(self.checkbox_communication_coppeliasim)
        spacerItem17 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem17)
        self.gridLayout_4.addLayout(self.horizontalLayout_5, 1, 2, 1, 1)
        spacerItem18 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Policy.Expanding, QtWidgets.QSizePolicy.Policy.Minimum)
        self.gridLayout_4.addItem(spacerItem18, 2, 1, 1, 1)
        self.frame_2 = QtWidgets.QFrame(self.frame_10)
        self.frame_2.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame_2.setObjectName("frame_2")
        self.gridLayout = QtWidgets.QGridLayout(self.frame_2)
        self.gridLayout.setObjectName("gridLayout")
        self.button_connect_port = QtWidgets.QPushButton(self.frame_2)
        self.button_connect_port.setObjectName("button_connect_port")
        self.gridLayout.addWidget(self.button_connect_port, 4, 0, 1, 2)
        self.combobox_port = QtWidgets.QComboBox(self.frame_2)
        self.combobox_port.setMinimumSize(QtCore.QSize(120, 0))
        self.combobox_port.setObjectName("combobox_port")
        self.gridLayout.addWidget(self.combobox_port, 1, 0, 1, 1)
        self.label_connection_status = QtWidgets.QLabel(self.frame_2)
        self.label_connection_status.setObjectName("label_connection_status")
        self.gridLayout.addWidget(self.label_connection_status, 3, 0, 1, 2)
        self.button_refresh_port = QtWidgets.QPushButton(self.frame_2)
        self.button_refresh_port.setObjectName("button_refresh_port")
        self.gridLayout.addWidget(self.button_refresh_port, 1, 1, 1, 1)
        spacerItem19 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout.addItem(spacerItem19, 2, 0, 1, 1)
        self.gridLayout_4.addWidget(self.frame_2, 2, 0, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.frame_10)
        self.label_3.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 0, 0, 1, 1)
        self.gridLayout_9.addWidget(self.frame_10, 4, 0, 3, 2)
        spacerItem20 = QtWidgets.QSpacerItem(20, 22, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_9.addItem(spacerItem20, 7, 0, 1, 1)
        spacerItem21 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_9.addItem(spacerItem21, 7, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.gridLayout_9.addWidget(self.label, 1, 2, 1, 1)
        self.frame_8 = QtWidgets.QFrame(self.centralwidget)
        self.frame_8.setFrameShape(QtWidgets.QFrame.Shape.StyledPanel)
        self.frame_8.setFrameShadow(QtWidgets.QFrame.Shadow.Raised)
        self.frame_8.setObjectName("frame_8")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.frame_8)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.textedit_status = QtWidgets.QTextEdit(self.frame_8)
        self.textedit_status.setEnabled(False)
        self.textedit_status.setObjectName("textedit_status")
        self.verticalLayout_2.addWidget(self.textedit_status)
        self.gridLayout_9.addWidget(self.frame_8, 4, 2, 3, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1197, 21))
        self.menubar.setObjectName("menubar")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "UMI Robot Control Console"))
        self.label_manual_control_angle_change.setText(_translate("MainWindow", "Angle change per click"))
        self.label_4.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:600;\">Manual Control</span></p></body></html>"))
        self.label_5.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:600;\">Teleoperation</span></p></body></html>"))
        self.label_6.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:600;\">Potentiometer Cal</span></p></body></html>"))
        self.label_2.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:600;\">Joint Values</span></p></body></html>"))
        self.label_7.setText(_translate("MainWindow", "<html><head/><body><p><span style=\" font-weight:600;\">UMI Robot</span></p></body></html>"))
        self.button_increase_q_1.setText(_translate("MainWindow", ">>"))
        self.button_decrease_q_4.setText(_translate("MainWindow", "<<"))
        self.button_decrease_q_5.setText(_translate("MainWindow", "<<"))
        self.button_decrease_q_1.setText(_translate("MainWindow", "<<"))
        self.button_increase_q_5.setText(_translate("MainWindow", ">>"))
        self.button_decrease_q_3.setText(_translate("MainWindow", "<<"))
        self.button_decrease_q_0.setText(_translate("MainWindow", "<<"))
        self.button_increase_q_0.setText(_translate("MainWindow", ">>"))
        self.button_decrease_q_2.setText(_translate("MainWindow", "<<"))
        self.button_increase_q_2.setText(_translate("MainWindow", ">>"))
        self.button_increase_q_3.setText(_translate("MainWindow", ">>"))
        self.button_increase_q_4.setText(_translate("MainWindow", ">>"))
        self.label_q_0.setText(_translate("MainWindow", "q0"))
        self.label_q_1.setText(_translate("MainWindow", "q1"))
        self.label_q_2.setText(_translate("MainWindow", "q2"))
        self.label_q_3.setText(_translate("MainWindow", "q3"))
        self.label_q_4.setText(_translate("MainWindow", "q4"))
        self.label_q_5.setText(_translate("MainWindow", "q5"))
        self.label_pot_0.setText(_translate("MainWindow", "qd0"))
        self.label_pot_2.setText(_translate("MainWindow", "qd2"))
        self.label_pot_3.setText(_translate("MainWindow", "qd3"))
        self.label_pot_5.setText(_translate("MainWindow", "qd5"))
        self.label_pot_1.setText(_translate("MainWindow", "qd1"))
        self.label_pot_4.setText(_translate("MainWindow", "qd4"))
        self.label_13.setText(_translate("MainWindow", "Communication"))
        self.checkbox_communication_arduino_all.setText(_translate("MainWindow", "All"))
        self.checkbox_communication_arduino_master_only.setText(_translate("MainWindow", "Master only"))
        self.label_11.setText(_translate("MainWindow", "IP"))
        self.lineedit_vrep_ip.setText(_translate("MainWindow", "127.0.0.1"))
        self.label_10.setText(_translate("MainWindow", "Port"))
        self.lineedit_vrep_port.setText(_translate("MainWindow", "20000"))
        self.label_connection_status_vrep.setText(_translate("MainWindow", "Disconnected."))
        self.button_connect_vrep.setText(_translate("MainWindow", "Connect"))
        self.label_9.setText(_translate("MainWindow", "CoppeliaSim"))
        self.button_connect_port.setText(_translate("MainWindow", "Connect"))
        self.label_connection_status.setText(_translate("MainWindow", "Disconnected."))
        self.button_refresh_port.setText(_translate("MainWindow", "Refresh"))
        self.label_3.setText(_translate("MainWindow", "Arduino"))
        self.label.setText(_translate("MainWindow", "Information"))



import multiprocessing
from umirobot import umirobot_communication_loop
from umirobot.gui import UMIRobotMainWindow

if __name__ == '__main__':
  multiprocessing.freeze_support()
  umirobot_communication_loop(UMIRobotMainWindow.run) 



"""
Copyright (C) 2020-2022 Murilo Marques Marinho (www.murilomarinho.info)
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program. If not,
see <https://www.gnu.org/licenses/>.
"""
from umirobot.shared_memory.umirobot_shared_memory_common import shared_memory_map


class UMIRobotSharedMemoryReceiver:
    def __init__(self, shared_memory_lists_tuple, lock):
        self.connection_information_dict = shared_memory_map
        self.connection_information, \
        self.shareable_q, \
        self.shareable_qd, \
        self.shareable_potentiometer_values,\
        self.shareable_digital_in_values = shared_memory_lists_tuple
        self.dofs = len(self.shareable_q)
        self.n_potentiometers = len(self.shareable_potentiometer_values)
        self.lock = lock

    def send_qd(self, qd):
        if qd is not None:
            if len(qd) == self.dofs:
                self.lock.acquire()
                for i in range(0, self.dofs):
                    self.shareable_qd[i] = qd[i]
                self.lock.release()

    def get_q(self):
        self.lock.acquire()
        q = list(self.shareable_q)
        self.lock.release()
        return q

    def get_potentiometer_values(self):
        self.lock.acquire()
        potentiometer_values = list(self.shareable_potentiometer_values)
        self.lock.release()
        return potentiometer_values

    def get_digital_in_values(self):
        self.lock.acquire()
        digital_in_values = list(self.shareable_digital_in_values)
        self.lock.release()
        return digital_in_values

    def is_open(self):
        self.lock.acquire()
        is_open = self.connection_information[self.connection_information_dict['is_open']]
        self.lock.release()
        return is_open

    def send_port(self, port):
        self.lock.acquire()
        if not self.connection_information[self.connection_information_dict['port_connect_signal']]:
            self.connection_information[self.connection_information_dict['port']] = port
            self.connection_information[self.connection_information_dict['port_connect_signal']] = True
        else:
            print("UMIRobotSharedMemoryReceiver::send_port::Unable to send port.")
        self.lock.release()

    def get_port(self):
        self.lock.acquire()
        port = self.connection_information[self.connection_information_dict['port']]
        self.lock.release()
        return port

    def send_shutdown_flag(self, flag):
        self.lock.acquire()
        self.connection_information[self.connection_information_dict['shutdown_flag']] = flag
        self.lock.release()

    def get_shutdown_flag(self):
        self.lock.acquire()
        shutdown_flag = self.connection_information[self.connection_information_dict['shutdown_flag']]
        self.lock.release()
        return shutdown_flag



"""
Copyright (C) 2020-2021 Murilo Marques Marinho (www.murilomarinho.info)
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program. If not,
see <https://www.gnu.org/licenses/>.
"""
import os

#  https://stackoverflow.com/questions/1854/python-what-os-am-i-running-on
from sys import platform as _platform

if _platform == 'win32':
    import ctypes
elif _platform == 'darwin':
    import glob
else:
    raise Exception('Unsupported platform ', _platform)

from PyQt6.QtCore import QTimer
from PyQt6.QtWidgets import QApplication, QMainWindow
from PyQt6.QtGui import QPixmap, QIcon

from dqrobotics.interfaces.vrep import DQ_VrepInterface
from dqrobotics.utils.DQ_Math import deg2rad, rad2deg

from umirobot.shared_memory import UMIRobotSharedMemoryReceiver
from umirobot.gui._umirobot_main_window_ui import Ui_MainWindow


# Regenerate UI with 'pyuic6 umirobot_main_window.ui -o _umirobot_main_window_ui.py'
class UMIRobotMainWindow(QMainWindow):
    def __init__(self, umi_robot_shared_memory_receiver):
        super(UMIRobotMainWindow, self).__init__()
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        self.statusBar().showMessage("Copyright (c) 2020-2022 Murilo M. Marinho (www.murilomarinho.info)")

        self.umi_robot_shared_memory_receiver = umi_robot_shared_memory_receiver
        self.is_open = False
        self.qd = [0, 0, 0, 0, 0, 0]
        self.q_min = -90
        self.q_max = 90

        self.pot_min = [None] * 6
        self.pot_max = [None] * 6
        self.potentiometer_calibration_difference = 3.0

        self.potentiometer_qd = [None] * 6

        self.vrep_interface = None
        self.vrep_ip = str(self.ui.lineedit_vrep_ip.text())
        self.vrep_port = int(self.ui.lineedit_vrep_port.text())

        # Fix robot figure not showing
        # https://stackoverflow.com/questions/33454555/bind-relative-image-path-to-py-file
        path = os.path.dirname(os.path.abspath(__file__))
        self.ui.label_image.setPixmap(QPixmap(os.path.join(path, 'umirobot.png')))
        self.setWindowIcon(QIcon(os.path.join(path, 'icon.png')))

        self.timer_ = QTimer()
        self.timer_.timeout.connect(self._timer_callback)
        self.timer_.start(1)

        # Increase buttons
        self.ui.button_increase_q_0.clicked.connect(self._button_increase_q_0_clicked_callback)
        self.ui.button_increase_q_1.clicked.connect(self._button_increase_q_1_clicked_callback)
        self.ui.button_increase_q_2.clicked.connect(self._button_increase_q_2_clicked_callback)
        self.ui.button_increase_q_3.clicked.connect(self._button_increase_q_3_clicked_callback)
        self.ui.button_increase_q_4.clicked.connect(self._button_increase_q_4_clicked_callback)
        self.ui.button_increase_q_5.clicked.connect(self._button_increase_q_5_clicked_callback)

        # Decrease buttons
        self.ui.button_decrease_q_0.clicked.connect(self._button_decrease_q_0_clicked_callback)
        self.ui.button_decrease_q_1.clicked.connect(self._button_decrease_q_1_clicked_callback)
        self.ui.button_decrease_q_2.clicked.connect(self._button_decrease_q_2_clicked_callback)
        self.ui.button_decrease_q_3.clicked.connect(self._button_decrease_q_3_clicked_callback)
        self.ui.button_decrease_q_4.clicked.connect(self._button_decrease_q_4_clicked_callback)
        self.ui.button_decrease_q_5.clicked.connect(self._button_decrease_q_5_clicked_callback)

        # Control
        self.ui.checkbox_jointcontrol.clicked.connect(self._checkbox_manual_control_clicked_callback)
        self.ui.slider_manual_control.valueChanged.connect(self._slider_manual_control_value_changed_callback)
        self.ui.label_manual_control_angle_change.setText(
            "Angle change per click " + str(self.ui.slider_manual_control.value()))

        self.ui.checkbox_teleoperation.clicked.connect(self._checkbox_teleoperation_clicked_callback)

        self.ui.checkbox_potentiometer_cal.clicked.connect(self._checkbox_potentiometer_cal_clicked_callback)

        # Connection related buttons
        self.ui.button_connect_port.clicked.connect(self._button_connect_port_clicked_callback)
        self.ui.button_refresh_port.clicked.connect(self._button_refresh_port_clicked_callback)
        self._button_refresh_port_clicked_callback()

        self.ui.button_connect_vrep.clicked.connect(self._button_connect_vrep_clicked_callback)
        self.ui.checkbox_communication_coppeliasim.clicked.connect(self._checkbox_communication_vrep_clicked_callback)
        self.ui.checkbox_communication_arduino_all.clicked.connect(
            self._checkbox_communication_arduino_all_clicked_callback)
        self.ui.checkbox_communication_arduino_master_only.clicked.connect(
            self._checkbox_communication_arduino_master_only_clicked_callback)

    def log(self, msg):
        self.ui.textedit_status.append(msg)

    def _button_connect_vrep_clicked_callback(self):
        if self.ui.checkbox_communication_coppeliasim.isChecked():
            if self.vrep_interface is None:
                try:
                    self.vrep_interface = DQ_VrepInterface()
                    self.vrep_ip = str(self.ui.lineedit_vrep_ip.text())
                    self.vrep_port = int(self.ui.lineedit_vrep_port.text())
                    if not self.vrep_interface.connect(self.vrep_ip, self.vrep_port, 100, 10):
                        raise RuntimeError("Unable to connect to {}:{}.".format(self.vrep_ip, self.vrep_port))
                except Exception as e:
                    self.vrep_interface.disconnect_all()
                    self.log("Error::_button_connect_vrep_pressed" + str(e))
                    self.vrep_interface = None

    def _get_manual_control_increment(self):
        return self.ui.slider_manual_control.value()

    def _communication_mode_switch_reset(self):
        self.is_open = False
        self.qd = [0, 0, 0, 0, 0, 0]
        self.q = [None] * 6

    def _checkbox_communication_arduino_all_clicked_callback(self):
        if self.ui.checkbox_communication_arduino_all.isChecked():
            self._reset_control_modes()
            self.log("Info::Switched mode to Arduino communication (All).")
            self.ui.checkbox_communication_coppeliasim.setChecked(False)
            self.ui.checkbox_communication_arduino_master_only.setChecked(False)
            self._communication_mode_switch_reset()
            if self.vrep_interface is not None:
                self.vrep_interface.disconnect()
                self.vrep_interface = None

    def _checkbox_communication_arduino_master_only_clicked_callback(self):
        if self.ui.checkbox_communication_arduino_master_only.isChecked():
            self._reset_control_modes()
            self.log("Info::Switched mode to Arduino communication (Master only).")
            self.ui.checkbox_communication_arduino_all.setChecked(False)
            self._communication_mode_switch_reset()

    def _checkbox_communication_vrep_clicked_callback(self):
        if self.ui.checkbox_communication_coppeliasim.isChecked():
            self.log("Info::Switched mode to CoppeliaSim communication.")
            self.ui.checkbox_communication_arduino_all.setChecked(False)
            self._communication_mode_switch_reset()

    def _slider_manual_control_value_changed_callback(self):
        self.ui.label_manual_control_angle_change.setText(
            "Angle change per click " + str(self.ui.slider_manual_control.value()))

    def _button_increase_q_0_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[0] = max(self.q_min, min(self.qd[0] + self._get_manual_control_increment(), self.q_max))

    def _button_increase_q_1_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[1] = max(self.q_min, min(self.qd[1] + self._get_manual_control_increment(), self.q_max))

    def _button_increase_q_2_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[2] = max(self.q_min, min(self.qd[2] + self._get_manual_control_increment(), self.q_max))

    def _button_increase_q_3_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[3] = max(self.q_min, min(self.qd[3] + self._get_manual_control_increment(), self.q_max))

    def _button_increase_q_4_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[4] = max(self.q_min, min(self.qd[4] + self._get_manual_control_increment(), self.q_max))

    def _button_increase_q_5_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[5] = max(self.q_min, min(self.qd[5] + self._get_manual_control_increment(), self.q_max))

    def _button_decrease_q_0_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[0] = max(self.q_min, min(self.qd[0] - self._get_manual_control_increment(), self.q_max))

    def _button_decrease_q_1_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[1] = max(self.q_min, min(self.qd[1] - self._get_manual_control_increment(), self.q_max))

    def _button_decrease_q_2_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[2] = max(self.q_min, min(self.qd[2] - self._get_manual_control_increment(), self.q_max))

    def _button_decrease_q_3_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[3] = max(self.q_min, min(self.qd[3] - self._get_manual_control_increment(), self.q_max))

    def _button_decrease_q_4_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[4] = max(self.q_min, min(self.qd[4] - self._get_manual_control_increment(), self.q_max))

    def _button_decrease_q_5_clicked_callback(self):
        if self._get_manual_control_state():
            self.qd[5] = max(self.q_min, min(self.qd[5] - self._get_manual_control_increment(), self.q_max))

    def _get_manual_control_state(self):
        return self.ui.checkbox_jointcontrol.isChecked()

    def _get_teleoperation_state(self):
        return self.ui.checkbox_teleoperation.isChecked()

    def _reset_control_modes(self):
        self.log("Info::Reset control modes.")
        self.ui.checkbox_teleoperation.setChecked(False)
        self.ui.checkbox_potentiometer_cal.setChecked(False)
        self.ui.checkbox_jointcontrol.setChecked(False)

    def _checkbox_manual_control_clicked_callback(self):
        if self.ui.checkbox_jointcontrol.isChecked():
            self.log("Info::Switched mode to manual control.")
            self.ui.checkbox_teleoperation.setChecked(False)
            self.ui.checkbox_potentiometer_cal.setChecked(False)

    def _checkbox_teleoperation_clicked_callback(self):
        if self.ui.checkbox_teleoperation.isChecked():
            self.log("Info::Switched mode to teleoperation.")
            self.ui.checkbox_jointcontrol.setChecked(False)
            self.ui.checkbox_potentiometer_cal.setChecked(False)

    def _checkbox_potentiometer_cal_clicked_callback(self):
        if self.ui.checkbox_potentiometer_cal.isChecked():
            self.log("Info::Switched mode to potentiometer calibration.")
            self.ui.checkbox_jointcontrol.setChecked(False)
            self.ui.checkbox_teleoperation.setChecked(False)

    def _timer_callback(self):
        # Connection status (CoppeliaSim)
        if self.vrep_interface is None:
            self.ui.label_connection_status_vrep.setText("Disconnected.")
        else:
            self.ui.label_connection_status_vrep.setText("Connected.")

        # CoppeliaSim communication
        if self.ui.checkbox_communication_coppeliasim.isChecked():
            if self.vrep_interface is not None:

                try:
                    vrep_joint_names = ['UMIRobot_joint_1',
                                        'UMIRobot_joint_2',
                                        'UMIRobot_joint_3',
                                        'UMIRobot_joint_4',
                                        'UMIRobot_joint_5',
                                        'UMIRobot_joint_6']
                    q = rad2deg(self.vrep_interface.get_joint_positions(vrep_joint_names))
                    self.ui.lineedit_q_0.setText(f"{q[0]:.1f}")
                    self.ui.lineedit_q_1.setText(f"{q[1]:.1f}")
                    self.ui.lineedit_q_2.setText(f"{q[2]:.1f}")
                    self.ui.lineedit_q_3.setText(f"{q[3]:.1f}")
                    self.ui.lineedit_q_4.setText(f"{q[4]:.1f}")
                    self.ui.lineedit_q_5.setText(f"{q[5]:.1f}")

                    self.vrep_interface.set_joint_target_positions(vrep_joint_names, deg2rad(self.qd))

                except Exception as e:
                    self.log("Error::_timer_callback" + str(e))

        if self.ui.checkbox_communication_arduino_all.isChecked() or \
                self.ui.checkbox_communication_arduino_master_only.isChecked():
            # Arduino communication
            current_serial_is_open = self.umi_robot_shared_memory_receiver.is_open()
            # Serial port communication
            if current_serial_is_open is None:
                self.log(
                    "Warning::Error reading shared memory. current_serial_is_open = {}".format(current_serial_is_open))
                return

            if self.is_open and not current_serial_is_open:
                self.log("Warning::Connection lost to {}.".format(self.umi_robot_shared_memory_receiver.get_port()))
                self._button_refresh_port_clicked_callback()
            if (not self.is_open) and current_serial_is_open:
                self.log("Info::Connection established at {}.".format(self.umi_robot_shared_memory_receiver.get_port()))

            if current_serial_is_open:
                self.is_open = True
                self.ui.label_connection_status.setText(
                    "Connected to {}.".format(self.umi_robot_shared_memory_receiver.get_port()))
                try:
                    if self.ui.checkbox_communication_arduino_all.isChecked():
                        q = self.umi_robot_shared_memory_receiver.get_q()
                        self.ui.lineedit_q_0.setText(str(q[0]))
                        self.ui.lineedit_q_1.setText(str(q[1]))
                        self.ui.lineedit_q_2.setText(str(q[2]))
                        self.ui.lineedit_q_3.setText(str(q[3]))
                        self.ui.lineedit_q_4.setText(str(q[4]))
                        self.ui.lineedit_q_5.setText(str(q[5]))

                    potentiometer_values = self.umi_robot_shared_memory_receiver.get_potentiometer_values()

                    if self._are_potentiometers_calibrated():
                        for i in range(0, 6):
                            self.potentiometer_qd[i] = int(
                                float(potentiometer_values[i] / (self.pot_max[i] - self.pot_min[i])) * 180.0) - 90

                    self.ui.lineedit_teleop_0.setText(str(self.potentiometer_qd[0]))
                    self.ui.lineedit_teleop_1.setText(str(self.potentiometer_qd[1]))
                    self.ui.lineedit_teleop_2.setText(str(self.potentiometer_qd[2]))
                    self.ui.lineedit_teleop_4.setText(str(self.potentiometer_qd[3]))
                    self.ui.lineedit_teleop_3.setText(str(self.potentiometer_qd[4]))
                    self.ui.lineedit_teleop_5.setText(str(self.potentiometer_qd[5]))

                    if potentiometer_values[0] is not None:
                        self.ui.lineedit_pot_0.setText("{0:.3g} V".format(potentiometer_values[0]))
                    if potentiometer_values[1] is not None:
                        self.ui.lineedit_pot_1.setText("{0:.3g} V".format(potentiometer_values[1]))
                    if potentiometer_values[2] is not None:
                        self.ui.lineedit_pot_2.setText("{0:.3g} V".format(potentiometer_values[2]))
                    if potentiometer_values[3] is not None:
                        self.ui.lineedit_pot_3.setText("{0:.3g} V".format(potentiometer_values[3]))
                    if potentiometer_values[4] is not None:
                        self.ui.lineedit_pot_4.setText("{0:.3g} V".format(potentiometer_values[4]))
                    if potentiometer_values[5] is not None:
                        self.ui.lineedit_pot_5.setText("{0:.3g} V".format(potentiometer_values[5]))

                    if self.pot_min[0] is not None:
                        self.ui.lineedit_pot_min_0.setText("{0:.3g} V".format(self.pot_min[0]))
                    if self.pot_min[1] is not None:
                        self.ui.lineedit_pot_min_1.setText("{0:.3g} V".format(self.pot_min[1]))
                    if self.pot_min[2] is not None:
                        self.ui.lineedit_pot_min_2.setText("{0:.3g} V".format(self.pot_min[2]))
                    if self.pot_min[3] is not None:
                        self.ui.lineedit_pot_min_3.setText("{0:.3g} V".format(self.pot_min[3]))
                    if self.pot_min[4] is not None:
                        self.ui.lineedit_pot_min_4.setText("{0:.3g} V".format(self.pot_min[4]))
                    if self.pot_min[5] is not None:
                        self.ui.lineedit_pot_min_5.setText("{0:.3g} V".format(self.pot_min[5]))

                    if self.pot_max[0] is not None:
                        self.ui.lineedit_pot_max_0.setText("{0:.3g} V".format(self.pot_max[0]))
                    if self.pot_max[1] is not None:
                        self.ui.lineedit_pot_max_1.setText("{0:.3g} V".format(self.pot_max[1]))
                    if self.pot_max[2] is not None:
                        self.ui.lineedit_pot_max_2.setText("{0:.3g} V".format(self.pot_max[2]))
                    if self.pot_max[3] is not None:
                        self.ui.lineedit_pot_max_3.setText("{0:.3g} V".format(self.pot_max[3]))
                    if self.pot_max[4] is not None:
                        self.ui.lineedit_pot_max_4.setText("{0:.3g} V".format(self.pot_max[4]))
                    if self.pot_max[5] is not None:
                        self.ui.lineedit_pot_max_5.setText("{0:.3g} V".format(self.pot_max[5]))

                    if self.ui.checkbox_teleoperation.isChecked():
                        if self._are_potentiometers_calibrated():
                            for i in range(0, 6):
                                self.qd[i] = self.potentiometer_qd[i]

                    if self.ui.checkbox_potentiometer_cal.isChecked():
                        for i in range(0, 6):
                            if self.pot_min[i] is None:
                                self.pot_min[i] = potentiometer_values[i]
                            if self.pot_max[i] is None:
                                self.pot_max[i] = potentiometer_values[i]

                        self.pot_min[0] = min(self.pot_min[0], potentiometer_values[0])
                        self.pot_min[1] = min(self.pot_min[1], potentiometer_values[1])
                        self.pot_min[2] = min(self.pot_min[2], potentiometer_values[2])
                        self.pot_min[3] = min(self.pot_min[3], potentiometer_values[3])
                        self.pot_min[4] = min(self.pot_min[4], potentiometer_values[4])
                        self.pot_min[5] = min(self.pot_min[5], potentiometer_values[5])

                        self.pot_max[0] = max(self.pot_max[0], potentiometer_values[0])
                        self.pot_max[1] = max(self.pot_max[1], potentiometer_values[1])
                        self.pot_max[2] = max(self.pot_max[2], potentiometer_values[2])
                        self.pot_max[3] = max(self.pot_max[3], potentiometer_values[3])
                        self.pot_max[4] = max(self.pot_max[4], potentiometer_values[4])
                        self.pot_max[5] = max(self.pot_max[5], potentiometer_values[5])

                        if abs(self.pot_min[0] - self.pot_max[0]) > self.potentiometer_calibration_difference:
                            self.ui.radio_pot_0.setChecked(True)
                        if abs(self.pot_min[1] - self.pot_max[1]) > self.potentiometer_calibration_difference:
                            self.ui.radio_pot_1.setChecked(True)
                        if abs(self.pot_min[2] - self.pot_max[2]) > self.potentiometer_calibration_difference:
                            self.ui.radio_pot_2.setChecked(True)
                        if abs(self.pot_min[3] - self.pot_max[3]) > self.potentiometer_calibration_difference:
                            self.ui.radio_pot_3.setChecked(True)
                        if abs(self.pot_min[4] - self.pot_max[4]) > self.potentiometer_calibration_difference:
                            self.ui.radio_pot_4.setChecked(True)
                        if abs(self.pot_min[5] - self.pot_max[5]) > self.potentiometer_calibration_difference:
                            self.ui.radio_pot_5.setChecked(True)

                    if self.ui.checkbox_communication_arduino_all.isChecked():
                        self.umi_robot_shared_memory_receiver.send_qd(self.qd)
                    else:
                        self.umi_robot_shared_memory_receiver.send_qd([0, 0, 0, 0, 0, 0])
                except Exception as e:
                    self.log("Error::_timer_callback" + str(e))
            else:
                self.is_open = False
                self.ui.label_connection_status.setText("Disconnected.")

    def _are_potentiometers_calibrated(self):
        if self.ui.radio_pot_0.isChecked() and \
                self.ui.radio_pot_1.isChecked() and \
                self.ui.radio_pot_2.isChecked() and \
                self.ui.radio_pot_3.isChecked() and \
                self.ui.radio_pot_4.isChecked() and \
                self.ui.radio_pot_5.isChecked():
            return True
        else:
            return False

    def _button_connect_port_clicked_callback(self):
        if self.ui.checkbox_communication_arduino_all.isChecked() or \
                self.ui.checkbox_communication_arduino_master_only.isChecked():
            current_selected_port = self.ui.combobox_port.currentText()
            if (current_selected_port == self.umi_robot_shared_memory_receiver.get_port()) and \
                    self.umi_robot_shared_memory_receiver.is_open():
                self.log("Log::Already connected to {}.".format(current_selected_port))
                return
            if current_selected_port != "":
                self.umi_robot_shared_memory_receiver.send_port(current_selected_port)
                self.log("Info::Requesting connection to {}...".format(current_selected_port))
            else:
                self.log("Warning::No port selected.")
        else:
            self.log("Warning::No connection node selected.")

    def _button_refresh_port_clicked_callback(self):
        self.ui.combobox_port.clear()
        if _platform == 'win32':
            import serial.tools.list_ports
            ports_list = serial.tools.list_ports.comports()
            for port in ports_list:
                self.ui.combobox_port.addItem(str(port.name))
                if "Arduino Uno" in port.description:
                    self.log("Info::Arduino Uno found at {}.".format(port.name))
        elif _platform == 'darwin':
            # https://stackoverflow.com/questions/22467683/listing-serial-ports-in-mac-os-x-and-python-3
            ports_list = glob.glob('/dev/tty.*')
            for port in ports_list:
                self.ui.combobox_port.addItem(str(port))
        if len(ports_list) == 0:
            self.log("Warning::Updated ports but no ports available.")
        else:
            self.log("Info::Found {} serial ports.".format(len(ports_list)))

    @staticmethod
    def run(shared_memory_info, lock):
        umi_robot_shared_memory_receiver = UMIRobotSharedMemoryReceiver(shared_memory_info, lock)
        app = QApplication([])
        myapp = UMIRobotMainWindow(umi_robot_shared_memory_receiver)

        # Set app icon
        path = os.path.dirname(os.path.abspath(__file__))
        app.setWindowIcon(QIcon(os.path.join(path, 'icon.png')))

        # Set theme for Windows and fix icon not showing on taskbar
        if _platform == 'win32':
            # Palette from this link modified to PyQt6 by myself
            # https://stackoverflow.com/questions/15035767/is-the-qt-5-dark-fusion-theme-available-for-windows
            from PyQt6.QtWidgets import QStyleFactory
            from PyQt6.QtGui import QPalette, QColor
            from PyQt6.QtCore import Qt
            app.setStyle(QStyleFactory.create("Fusion"))
            defaultFont = QApplication.font()
            defaultFont.setPointSize(defaultFont.pointSize() + 2)
            app.setFont(defaultFont)
            dark_palette = QPalette()
            dark_palette.setColor(QPalette.ColorRole.Window, QColor(53, 53, 53))
            dark_palette.setColor(QPalette.ColorRole.WindowText, Qt.GlobalColor.white)
            dark_palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.WindowText, QColor(127, 127, 127))
            dark_palette.setColor(QPalette.ColorRole.Base, QColor(42, 42, 42))
            dark_palette.setColor(QPalette.ColorRole.AlternateBase, QColor(66, 66, 66))
            dark_palette.setColor(QPalette.ColorRole.ToolTipBase, Qt.GlobalColor.white)
            dark_palette.setColor(QPalette.ColorRole.ToolTipText, Qt.GlobalColor.white)
            dark_palette.setColor(QPalette.ColorRole.Text, Qt.GlobalColor.white)
            dark_palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.Text, QColor(127, 127, 127))
            dark_palette.setColor(QPalette.ColorRole.Dark, QColor(35, 35, 35))
            dark_palette.setColor(QPalette.ColorRole.Shadow, QColor(20, 20, 20))
            dark_palette.setColor(QPalette.ColorRole.Button, QColor(53, 53, 53))
            dark_palette.setColor(QPalette.ColorRole.ButtonText, Qt.GlobalColor.white)
            dark_palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.ButtonText, QColor(127, 127, 127))
            dark_palette.setColor(QPalette.ColorRole.BrightText, Qt.GlobalColor.red)
            dark_palette.setColor(QPalette.ColorRole.Link, QColor(42, 130, 218))
            dark_palette.setColor(QPalette.ColorRole.Highlight, QColor(42, 130, 218))
            dark_palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.Highlight, QColor(80, 80, 80))
            dark_palette.setColor(QPalette.ColorRole.HighlightedText, Qt.GlobalColor.white)
            dark_palette.setColor(QPalette.ColorGroup.Disabled, QPalette.ColorRole.HighlightedText, QColor(127, 127, 127))

            app.setPalette(dark_palette)

            # https://stackoverflow.com/questions/1551605/how-to-set-applications-taskbar-icon-in-windows-7
            ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID('utokyo.umirobot')

        myapp.show()

        try:
            app.exec()
        except Exception as e:
            print("umirobot_main_window::run::Error::" + str(e))
        except KeyboardInterrupt:
            print("umirobot_main_window::run::Info::Interrupted by user.")

        umi_robot_shared_memory_receiver.send_shutdown_flag(True)



"""
Copyright (C) 2020-2022 Murilo Marques Marinho (www.murilomarinho.info)
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program. If not,
see <https://www.gnu.org/licenses/>.
"""
# TODO add the default values here as well
shared_memory_map = {
    "n_servos": 0,
    "n_potentiometers": 1,
    "n_digital_inputs": 2,
    "is_open": 3,
    "port": 4,
    "port_connect_signal": 5,
    "shutdown_flag": 6
}



#!/usr/bin/env python
"""
Copyright (C) 2020-2022 Murilo Marques Marinho (www.murilomarinho.info)
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program. If not,
see <https://www.gnu.org/licenses/>.
"""
import multiprocessing as mp
import multiprocessing.managers as mm

from umirobot import UMIRobot
from umirobot.shared_memory import UMIRobotSharedMemoryProvider


def umirobot_communication_loop(run):
    with UMIRobot() as umirobot, mm.SharedMemoryManager() as smm:
        # Lock
        lock = mp.Lock()
        # Provider
        shared_memory_provider = UMIRobotSharedMemoryProvider(shared_memory_manager=smm, lock=lock)
        # Receiver
        shared_memory_receiver_process = mp.Process(
            target=run,
            args=(shared_memory_provider.get_shared_memory_receiver_initializer_args(), lock)
        )
        shared_memory_receiver_process.start()

        try:
            # Control loop
            while True:

                # Connect to the serial port if requested
                if shared_memory_provider.get_port() is not None:
                    if shared_memory_provider.get_port_connect_signal():
                        if shared_memory_provider.get_port() != umirobot.get_port():
                            umirobot.set_port(shared_memory_provider.get_port())
                            shared_memory_provider.send_port_connect_signal(False)

                # If connection is open, update, handle q and qd
                if umirobot.is_open():
                    umirobot.update()
                    shared_memory_provider.send_q(umirobot.get_q())
                    shared_memory_provider.send_potentiometer_values(umirobot.get_potentiometer_values())
                    shared_memory_provider.send_digital_in_values(umirobot.get_digital_in_values())
                    umirobot.set_qd(shared_memory_provider.get_qd())

                # Always send connection status
                shared_memory_provider.send_is_open(umirobot.is_open())

                # Check if shutdown signal was sent by receiver
                if shared_memory_provider.get_shutdown_flag():
                    print('umirobot::__main__::Info::Provider shutdown by receiver.')
                    break

        except Exception as e:
            print('umirobot::__main__::Error::' + str(e))
        except KeyboardInterrupt:
            print('umirobot::__main__::Info::Shutdown by CTRL+C.')
        shared_memory_receiver_process.join()



"""
Copyright (C) 2021 Murilo Marques Marinho (www.murilomarinho.info)
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
version.
This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with this program. If not,
see <https://www.gnu.org/licenses/>.
"""
import setuptools
from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='umirobot',
    version_format='{tag}.{commitcount}',
    setup_requires=['setuptools-git-version'],
    install_requires=[
        'pyserial', 'PyQt6', 'dqrobotics'
    ],
    packages=setuptools.find_packages(),
    url='https://github.com/mmmarinho/umirobot-py',
    license='GPLv3',
    author='Murilo M. Marinho',
    author_email='murilo@g.ecc.u-tokyo.ac.jp',
    description='UMIRobot control code and GUI for Python.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires='>=3.8',
    package_data={'umirobot.gui': ['icon.png', 'umirobot.png']},
    include_package_data=True
)
